@@ -6,11 +6,12 @@ import json
 
 class Map(Field):
     def __init__(self, *args, **kws):
+
         default = {
             'left': None,
             'right': None,
             'meta': 'left',
-            'default': {},
+            'default': torm.utl.Map.Map({}),
             'field_type': 'dict',
             'key': False,
             'only_db_types': None
@@ -27,9 +28,9 @@ class Map(Field):
     def validate(self, value):
         model = self.model
         key = self.name
-        if type(value) != torm.utl.Map:
-            raise error_type(key, value, model, torm.utl.Map)
-        return True
+        if type(value) == torm.utl.Map.Map or type(value) == dict:
+            return True
+        raise error_type(key, value, model, torm.utl.Map)
 
     def to_json(self):
         return json.dumps(self.value)
@@ -37,3 +38,4 @@ class Map(Field):
     def from_json(self, json_str):
         self.value = json.loads(json_str)
         return self
+
